module ComplexField
  module SubjectIndexer
    def generate_solr_document
      super.tap do |solr_doc|
        index_subject(solr_doc)
      end
    end

    def index_subject(solr_doc)
      solr_doc[Solrizer.solr_name('complex_subject', :displayable)] = object.complex_subjects.to_json
      object.complex_subjects.each do |c|
        # species
        vals = [c.subject_species]
        fld_name = Solrizer.solr_name('complex_subject_species', :stored_searchable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
        fld_name = Solrizer.solr_name('complex_subject_species', :facetable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq

        # type
        vals = [c.subject_type]
        fld_name = Solrizer.solr_name('complex_subject_type', :stored_searchable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
        fld_name = Solrizer.solr_name('complex_subject_type', :facetable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq

        # sex
        vals = [c.subject_sex]
        fld_name = Solrizer.solr_name('complex_subject_sex', :stored_searchable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
        fld_name = Solrizer.solr_name('complex_subject_sex', :facetable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq

        # age
        vals = [c.subject_age]
        fld_name = Solrizer.solr_name('complex_subject_age', :stored_searchable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
        fld_name = Solrizer.solr_name('complex_subject_age', :facetable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
        # for integer range queries
        fld_name = 'complex_subject_age_itsim'
        int_vals = []
        vals.each do |v|
          int_vals << Integer(v) if Integer === Integer(v, exception: false)
        end
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << int_vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
      end
    end

    def self.subject_facet_fields
      # solr fields that will be treated as facets
      fields = []
      fields << Solrizer.solr_name('complex_subject_species', :facetable)
      fields << Solrizer.solr_name('complex_subject_type', :facetable)
      fields << Solrizer.solr_name('complex_subject_sex', :facetable)
      fields << 'complex_subject_age_itsim'
      fields
    end

    def self.subject_search_fields
      # solr fields that will be used for a search
      fields = []
      fields << Solrizer.solr_name('complex_subject_species', :stored_searchable)
      fields << Solrizer.solr_name('complex_subject_type', :stored_searchable)
      fields << Solrizer.solr_name('complex_subject_sex', :stored_searchable)
      fields << Solrizer.solr_name('complex_subject_age', :stored_searchable)
      fields
    end

    def self.subject_show_fields
      # solr fields that will be used to display results on the record page
      fields = []
      fields << Solrizer.solr_name('complex_subject', :displayable)
      fields
    end
  end
end
